/*



*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>

#include "dvb.h"
#include "parse_dvb.h"
#include "dump_dvb.h"

//----------------------------------------------------------------------------
//#define DESCLEN 0x40
//typedef struct
//{
//    int   id;
//    char  description[DESCLEN];
//} EnumType;



EnumType descriptor_str[] = {
    { 0x44, "unknown"},
    { 0x02, "VIDEO"},
    { 0x03, "AUDIO"},
    { 0x04, "HIERARCHY"},
    { 0x05, "REGISTRATION"},
    { 0x06, "DATA STREAM ALIGNMENT"},
    { 0x07, "TARGET BACKGROUND GRID"},
    { 0x08, "VIDEO WINDOW"},
    { 0x09, "CA"},
    { 0x0a, "ISO_639 LANGUAGE"},
    { 0x0b, "SYSTEM CLOCK"},
    { 0x0c, "MPLEX BUFFER UTILISATION"},
    { 0x0d, "COPYRIGHT"},
    { 0x0e, "MAXIMUM BITRATE"},
    { 0x0f, "PRIVATE DATA"},
    { 0x10, "SMOOTHING BUFFER"},
    { 0x11, "STD"},
    { 0x12, "IBP"},
    { 0x13, "CAROUSEL IDENTIFIER"},
    { 0x14, "ASSOCIATION TAG"},
    { 0x15, "DEFERRED ASSOCIATION TAGS"},
    { 0x40, "NETWORK NAME"},
    { 0x41, "SERVICE LIST"},
    { 0x42, "STUFFING"},
    { 0x43, "SATELLITE DELIVERY SYSTEM"},
    { 0x44, "CABLE DELIVERY SYSTEM"},
    { 0x45, "RESERVED"},
    { 0x46, "RESERVED"},
    { 0x47, "BOUQUET NAME"},
    { 0x48, "SERVICE"},
    { 0x49, "COUNTRY AVAILABILITY"},
    { 0x4a, "LINKAGE"},
    { 0x4b, "NVOD REFERENCE"},
    { 0x4c, "TIME SHIFTED SERVICE"},
    { 0x4d, "SHORT EVENT"},
    { 0x4e, "EXTENDED EVENT"},
    { 0x4f, "TIME SHIFTED EVENT"},
    { 0x50, "COMPONENT"},
    { 0x51, "MOSAIC"},
    { 0x52, "STREAM IDENTIFIER"},
    { 0x53, "CA IDENTIFIER"},
    { 0x54, "CONTENT"},
    { 0x55, "PARENTAL RATING"},
    { 0x56, "TELETEXT"},
    { 0x57, "TELEPHONE"},
    { 0x58, "LOCAL TIME OFFSET"},
    { 0x59, "SUBTITLING"},
    { 0x5a, "TERRESTRIAL DELIVERY SYSTEM"},
    { 0x5b, "MULTILINGUAL NETWORK NAME"},
    { 0x5c, "MULTILINGUAL BOUQUET NAME"},
    { 0x5d, "MULTILINGUAL SERVICE NAME"},
    { 0x5e, "MULTILINGUAL COMPONENT"},
    { 0x5f, "PRIVATE DATA SPECIFIER"},
    { 0x60, "SERVICE MOVE"},
    { 0x61, "SHORT SMOOTHING BUFFER"},
    { 0x62, "FREQUENCY LIST"},
    { 0x63, "PARTIAL TRANSPORT STREAM"},
    { 0x64, "DATA BROADCAST"},
    { 0x65, "CA SYSTEM"},
    { 0x66, "DATA BROADCAST ID"},
    { 0x67, "TRANSPORT STREAM"},
    { 0x68, "DSNG"},
    { 0x69, "PDC"},
    { 0x6a, "AC3"},
    { 0x6b, "ANCILLARY"},
    { 0x6c, "CELL LIST"},
    { 0x6d, "CELL FREQUENCY LINK LIST"},
    { 0x6e, "ANNOUNCEMENT SUPPORT"},
    { 0x83, "UNKNOWN 0x83 (CHANNEL MAP)"},
};

//network_id
EnumType network_id_str[] = {
// types
    { 0x1b,   "reserved"            },
    { 0x0001, "Astra 19.2E"         },
    { 0x0002, "Astra 28.2E"         },
    { 0x0020, "Astra"               },
    { 0x0027, "Hispasat 30W"        },
    { 0x0028, "Hispasat 30W"        },
    { 0x0029, "Hispasat 30W"        },
    { 0x0030, "Canal Plus"          },
    { 0x0035, "Nethold"             },
    { 0x0037, "Stentor"             },
    { 0x0055, "Sirius"              },
    { 0x0055, "Deutsche Telekom"    },
    { 0x0073, "PanAmSat4 68.5E"     },
    { 0x007e, "Eutelsat 7E"         },
    { 0x0085, "Beta Technik"        },
    { 0x00a6, "ART"                 },
    { 0x00a7, "Globecast"           },
    { 0x00a8, "Foxtel"              },
    { 0x013e, "Eutelsat 13E"        },
    { 0x016e, "Eutelsat 16E"        },
    { 0x029e, "Eutelsat 29E"        },
    { 0x02be, "Arabsat"             },
    { 0x036e, "Eutelsat 36E"        },
    { 0x03e8, "Telia"               },
    { 0x048e, "Eutelsat 48E"        },
    { 0x0800, "Nilesat 101"         },
    { 0x0801, "Nilesat 101"         },
    { 0x233a, "UK DTT"              },
};

//ca_system_id
EnumType ca_system_id_str[] = {
// types
    { 0x1c,   "reserved"            },
    { 0x0000, "fta"                 },
    { 0x0100, "Seca"                },      // Canal Plus
    { 0x0200, "CCET"                },
    { 0x0300, "Deutsche Telecom"    },
    { 0x0400, "Eurodec"             },
    { 0x0500, "Viaccess"            },      // France Telecom
    { 0x0600, "Irdeto"              },
    { 0x0700, "Jerrold/GI"          },
    { 0x0800, "Matra"               },
    { 0x0900, "Videoguard"          },      // News Datacom
    { 0x0a00, "Nokia"               },
    { 0x0b00, "Conax"               },      // Norwegian Telekom
    { 0x0c00, "NTL"                 },
    { 0x0d00, "Cryptoworks"         },      // Philips
    { 0x0e00, "Power VU"            },      // Scientific Atlanta
    { 0x0f00, "Sony"                },
    { 0x1000, "Tandberg"            },
    { 0x1100, "Thomson"             },
    { 0x1200, "TV/Com"              },
    { 0x1300, "HPT"                 },
    { 0x1400, "HRT"                 },
    { 0x1500, "IBM"                 },
    { 0x1600, "Nera"                },
    { 0x1700, "Betacrypt"           },      // Beta Technik
    { 0x1800, "Nagravision"         },
    { 0x1900, "Titan"               },
    { 0x2000, "Telefonica"          },
    { 0x2100, "STENTOR"             },
};

//satellite delivery system, FEC_inner
EnumType fec_str[] = {
// types
    { 0x06, "reserved"                 },
    { 0x00, "reserved"                 },
    { 0x01, "1/2"                      },
    { 0x02, "2/3"                      },
    { 0x03, "3/4"                      },
    { 0x04, "5/6"                      },
    { 0x05, "7/8"                      }
};

//frequency list, coding type
EnumType coding_type_str[] = {
// types
    { 0x03, "not defined"              },
    { 0x01, "Satellite"                },
    { 0x02, "Cable"                    },
    { 0x03, "Terrestrial"              }
};

//private_data_specifier_values
EnumType private_data_specifier_str[] = {
// types
    { 0x1f,   "reserved"            },
    { 0x00000001, "SES"             },
    { 0x00000002, "BSkyB 1"         },
    { 0x00000003, "BSkyB 2"         },
    { 0x00000004, "BSkyB 3"         },
    { 0x00000005, "ARD/ZDX/ORF"     },
    { 0x00000006, "Nokia"           },
    { 0x00000007, "AT"              },
    { 0x00000010, "TPS"             },
    { 0x00000011, "Echostar"        },
    { 0x00000012, "Telia"           },
    { 0x00000025, "MTV Europe"      },
    { 0x000000be, "Beta Technik"    },
    { 0x000000c0, "Canal Plus"      },
    { 0x000000d0, "Dolby"           },
    { 0x000000e0, "ExpressVu"       },
    { 0x000000f0, "STENTOR"         },
    { 0x00000100, "OpenTV"          },
    { 0x0000233a, "ITC"             },
    { 0x00006000, "News Datacom"    },
    { 0x00006001, "NDC 1"           },
    { 0x00006002, "NDC 2"           },
    { 0x00006003, "NDC 3"           },
    { 0x00006004, "NDC 4"           },
    { 0x00006005, "NDC 5"           },
    { 0x00006006, "NDC 6"           },
    { 0x00362275, "Irdeto"          },
    { 0x004e544c, "NTL"             },
    { 0x00532d41, "Scientific Atlanta" },
    { 0x44414e59, "News Datacom"    },
    { 0x46524549, "News Datacom"    },
    { 0x53415053, "Scientific Atlanta" },
};

//data_broadcast
EnumType data_broadcast_id_str[] = {
// types
    { 0x09, "reserved"                      },
    { 0x00, "reserved"                      },
    { 0x01, "bit_pipe"                      },
    { 0x02, "asynchronous data stream"      },
    { 0x03, "synchronous data stream"       },
    { 0x04, "synchronised data stream"      },
    { 0x05, "multiprotocol encapsulation"   },
    { 0x06, "data carousel"                 },
    { 0x07, "object carousel"               },
    { 0x08, "DVB ATM streams"               },
};

//data_broadcast
EnumType carousel_type_id_str[] = {
// types
    { 0x02, "reserved"                      },
    { 0x01, "one layer carousel"            },
    { 0x02, "two layer carousel"            },
};

EnumType running_status_str[] = {
    { 4,    "---"          },
    { 1,    "Not Running " },
    { 2,    "Starts Soon " },
    { 3,    "Pausing "     },
    { 4,    "Running "     },
};

EnumType free_ca_str[] = {
    { 2,    "---"          },
    { 0,    "FTA "         },
    { 1,    "Encrypted "   },
};

EnumType no_yes_str[] = {
    { 2,    "---"         },
    { 0,    "No"          },
    { 1,    "Yes"         },
};
//----------------------------------------------------------------------------

//stream_content, component_type, description
#define NO_COMPONENT_TYPES      39
struct
{
    unsigned char   stream_content;
    unsigned char   component_type;
    unsigned char   description[0x100];
} component[NO_COMPONENT_TYPES] = {
    { 0x1,0x01,"video, 4:3 aspect ratio" },
    { 0x1,0x02,"video, 16:9 aspect ratio, with pan vectors" },
    { 0x1,0x03,"video, 16:9 aspect ratio, without pan vectors" },
    { 0x1,0x04,"video, >16:9 aspect ratio" },
    { 0x1,0x05,"video, 4:3 aspect ratio, 30Hz" },
    { 0x1,0x06,"video, 16:9 aspect ratio, with pan vectors, 30Hz" },
    { 0x1,0x07,"video, 16:9 aspect ratio, without pan vectors, 30Hz" },
    { 0x1,0x08,"video, >16:9 aspect ratio, 30Hz" },
    { 0x1,0x09,"high definition video, 4:3 aspect ratio" },
    { 0x1,0x0a,"high definition video, 16:9 aspect ratio, with pan vectors" },
    { 0x1,0x0b,"high definition video, 16:9 aspect ratio, without pan vectors" },
    { 0x1,0x0c,"high definition video, >16:9 aspect ratio" },
    { 0x1,0x0d,"high definition video, 4:3 aspect ratio, 30Hz" },
    { 0x1,0x0e,"high definition video, 16:9 aspect ratio, with pan vectors, 30Hz" },
    { 0x1,0x0f,"high definition video, 16:9 aspect ratio, without pan vectors, 30Hz" },
    { 0x1,0x10,"high definition video, >16:9 aspect ratio, 30Hz" },

    { 0x2,0x01,"audio, single mono channel" },
    { 0x2,0x02,"audio, dual mono channel" },
    { 0x2,0x03,"audio, stereo (2 channel)" },
    { 0x2,0x04,"audio, multi-lingual, multi-channel" },
    { 0x2,0x05,"audio, surround sound" },
    { 0x2,0x40,"audio description for the visually impaired" },
    { 0x2,0x41,"audio for the hard of hearing" },

    { 0x3,0x01,"EBU Teletext subtitles" },
    { 0x3,0x02,"associated EBU Teletext" },
    { 0x3,0x03,"VBI data" },
    { 0x3,0x10,"audio, DVB subtitles (normal) with no monitor aspect ratio criticality" },
    { 0x3,0x11,"audio, DVB subtitles (normal) for display on 4:3 aspect ratio monitor" },
    { 0x3,0x12,"audio, DVB subtitles (normal) for display on 16:9 aspect ratio monitor" },
    { 0x3,0x13,"audio, DVB subtitles (normal) for display on 2.21:1 aspect ratio monitor" },
    { 0x3,0x20,"audio, DVB subtitles (for the hard of hearing) with no monitor aspect ratio criticality" },
    { 0x3,0x21,"audio, DVB subtitles (for the hard of hearing) for display on 4:3 aspect ratio monitor" },
    { 0x3,0x22,"audio, DVB subtitles (for the hard of hearing) for display on 16:9 aspect ratio monitor" },
    { 0x3,0x23,"audio, DVB subtitles (for the hard of hearing) for display on 2.21:1 aspect ratio monitor" },

    { 0x4,0x00,"AC3, Mono" },
    { 0x4,0x01,"AC3, 1+1 mode" },
    { 0x4,0x02,"AC3, 2 Channel (stereo)" },
    { 0x4,0x03,"AC3, 2 Channel Dolby surround encoded (stereo)" },
    { 0x4,0x04,"AC3, Multichannel audio (>2 channels)" },
};


//content_nibble1, content_nibble2
// level 2 does not appear to be used
#define NO_CONTENT_TYPES      81
struct
{
    unsigned char nibble1;
    unsigned char nibble2;
    unsigned char description[0x100];
} content[NO_CONTENT_TYPES] = {
    { 0x0,0x00,"undefined" },

    { 0x1,0x00,"movie/drama" },
    { 0x1,0x01,"detective/thriller" },
    { 0x1,0x02,"adventure/western/war" },
    { 0x1,0x03,"science fiction/fantasy/horror" },
    { 0x1,0x04,"comedy" },
    { 0x1,0x05,"soap/melodrama/folkloric" },
    { 0x1,0x06,"romance" },
    { 0x1,0x07,"serious/classical/religious/historical movie/drama" },
    { 0x1,0x08,"adult movie/drama" },

    { 0x2,0x00,"news/current affairs" },
    { 0x2,0x01,"news/weather report" },
    { 0x2,0x02,"news magazine" },
    { 0x2,0x03,"documentary" },
    { 0x2,0x04,"discussion/interview/debate" },

    { 0x3,0x00,"show/game show" },
    { 0x3,0x01,"game show/quiz/contest" },
    { 0x3,0x02,"variety show" },
    { 0x3,0x03,"talk show" },

    { 0x4,0x00,"sports" },
    { 0x4,0x01,"special sports events" },
    { 0x4,0x02,"sports magazines" },
    { 0x4,0x03,"football/soccer" },
    { 0x4,0x04,"tennis/squash" },
    { 0x4,0x05,"team sports" },
    { 0x4,0x06,"athletics" },
    { 0x4,0x07,"motor sport" },
    { 0x4,0x08,"water sport" },
    { 0x4,0x09,"winter sports" },
    { 0x4,0x0a,"equestrian" },
    { 0x4,0x0b,"martial sports" },

    { 0x5,0x00,"children's/youth programmes" },
    { 0x5,0x01,"pre-school children's programmes" },
    { 0x5,0x02,"entertainment programmes for 6 to 14" },
    { 0x5,0x03,"entertainment programmes for 10 to 16" },
    { 0x5,0x04,"informational/educational/school programmes" },
    { 0x5,0x05,"cartoons/puppets" },

    { 0x6,0x00,"music/ballet/dance" },
    { 0x6,0x01,"rock/pop" },
    { 0x6,0x02,"serious music/classical music" },
    { 0x6,0x03,"folk/traditional music" },
    { 0x6,0x04,"jazz" },
    { 0x6,0x05,"musical/opera" },
    { 0x6,0x06,"ballet" },

    { 0x7,0x00,"arts/culture (without music)" },
    { 0x7,0x01,"performing arts" },
    { 0x7,0x02,"fine arts" },
    { 0x7,0x03,"religion" },
    { 0x7,0x04,"popular culture/traditional arts" },
    { 0x7,0x05,"literature" },
    { 0x7,0x06,"film/cinema" },
    { 0x7,0x07,"experimental film/video" },
    { 0x7,0x08,"broadcasting/press" },
    { 0x7,0x09,"new media" },
    { 0x7,0x0a,"arts/culture magazines" },
    { 0x7,0x0b,"fashion" },

    { 0x8,0x00,"social/political issues/economics" },
    { 0x8,0x01,"magazines/reports/documentary" },
    { 0x8,0x02,"economics/social advisory" },
    { 0x8,0x03,"remarkable people" },

    { 0x9,0x00,"education/science/factual topics" },
    { 0x9,0x01,"nature/animals/environment" },
    { 0x9,0x02,"technology/natural sciences" },
    { 0x9,0x03,"medicine/physiology/psychology" },
    { 0x9,0x04,"foreign countries/expeditions" },
    { 0x9,0x05,"social/spiritual sciences" },
    { 0x9,0x06,"further education" },
    { 0x9,0x07,"languages" },

    { 0xa,0x00,"leisure hobbies" },
    { 0xa,0x01,"tourism/travel" },
    { 0xa,0x02,"handicraft" },
    { 0xa,0x03,"motoring" },
    { 0xa,0x04,"fitness & health" },
    { 0xa,0x05,"cooking" },
    { 0xa,0x06,"advertisement/shopping" },
    { 0xa,0x07,"gardening" },

    { 0xb,0x00,"original language" },
    { 0xb,0x01,"black and white" },
    { 0xb,0x02,"unpublished" },
    { 0xb,0x03,"live broadcast" },
    { 0xb,0x0f,"user defined" },
};


//----------------------------------------------------------------------------

int display_section_header(unsigned char *buf, unsigned char *extstr)
{
    int j;

    j = 0;
    printf("table_id                     = %02x\n", b70(buf, j++));
    printf("section_syntax_indicator     = %x\n", b7(buf, j));
//  printf("reserved0                    = %x\n", b6(buf, j));
//  printf("reserved1                    = %x\n", b54(buf, j));
    printf("section_length               = %03x\n", b110(buf, j));  j+=2;
    printf("%s          = %04x\n", extstr, b150(buf, j));  j+=2;
//  printf("reserved2                    = %x\n", b76(buf, j));
    printf("version_number               = %x\n", b51(buf, j));
    printf("current_next_indicator       = %x\n", b0(buf, j++));
    printf("section_number               = %02x\n", b70(buf, j++));
    printf("last_section_number          = %02x\n", b70(buf, j++));
    printf("\n");
    return j;
}


static char dflt[0x20];
char *get_component_type_str(int scontent, int ctype)
{
    int     i;

    for(i=0; i<NO_COMPONENT_TYPES; i++) {
        if ((component[i].stream_content == scontent) && (component[i].component_type == ctype)) {
            return component[i].description;
        }
    }
    sprintf(dflt,"UNKNOWN (0x%02x, 0x%02x)\0",scontent,ctype);
    return dflt;
}

char *get_content_type_str(int nibble1, int nibble2)
{
    int     i;

    for(i=0; i<NO_CONTENT_TYPES; i++) {
        if ((content[i].nibble1 == nibble1) && (content[i].nibble2 == nibble2)){
            return content[i].description;
        }
    }
    sprintf(dflt,"UNKNOWN (0x%02x, 0x%02x)\0",nibble1,nibble2);
    return dflt;
}

char *day[] = {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"};
void mjd(FILE *Op, unsigned char *t)
{
    int mjd1,yd,md,y,m,d,k,w,wy,wn,wd;

    mjd1 = (t[0] << 8) + t[1];

    yd = (int)((mjd1 - 15078.2) / 365.25);
    md = (int)((mjd1 - 14956.1 - (int)(yd * 365.25)) / 30.6001);
    d  = mjd1 - 14956 - (int)(yd * 365.25) - (int)(md * 30.6001);
    if ((md == 14) || (md == 15)) k=1; else k=0;
    y = 1900 + yd + k;
    m = md - 1 + k * 12;
    wd = ((mjd1 + 2) % 7) + 1;

    w = (int)((mjd1/7) - 2144.64);
    wy = (int)((w * 28 / 1461) - 0.0079);
    wn = w - (int)((wy * 1461 / 28) + 0.41);

    fprintf(Op,"%s, %02d/%02d/%02d, %02x:%02x:%02x",day[wd],d,m,y,t[2],t[3],t[4]);
}

char mjdstr[100];
char *getmjd(unsigned char *t)
{
    int mjd1,yd,md,y,m,d,k,w,wy,wn,wd;

    mjd1 = (t[0] << 8) + t[1];

    yd = (int)((mjd1 - 15078.2) / 365.25);
    md = (int)((mjd1 - 14956.1 - (int)(yd * 365.25)) / 30.6001);
    d  = mjd1 - 14956 - (int)(yd * 365.25) - (int)(md * 30.6001);
    if ((md == 14) || (md == 15)) k=1; else k=0;
    y = 1900 + yd + k;
    m = md - 1 + k * 12;
    wd = ((mjd1 + 2) % 7) + 1;

    w = (int)((mjd1/7) - 2144.64);
    wy = (int)((w * 28 / 1461) - 0.0079);
    wn = w - (int)((wy * 1461 / 28) + 0.41);

    sprintf(mjdstr,"%s, %02d/%02d/%02d, %02x:%02x:%02x",day[wd],d,m,y,t[2],t[3],t[4]);
    return mjdstr;
}


int dump_desc(unsigned char *pkt, int t, int s, int stream_type)
{
    int descriptor_tag, descriptor_length;
    int frequency, bandwidth;
    int i,j,k;
//    int polarisation, position;
//    int west_east_flag;
//    int symbol_rate;
//    int fec;
    int modulation;
    int fec_high, fec_low, hierarchy, guard_interval, transmission_mode;
    int service_type;
    int program_id_label;
    int ca_system_id;
    int ca_pid;
    char name[0x102];
	char text[0x102];

    j = 0;
    descriptor_tag = b70(pkt, j++);
    descriptor_length = b70(pkt, j++);
    if (!descriptor_length) return 0;
    switch (descriptor_tag)  {
        case VIDEO_STREAM_DESCRIPTOR:                   // 0x02
            printf("  VIDEO DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  multiple_frame_rate_flag   = %x\n",b7(pkt,j));
            printf("  frame_rate_code            = %x\n",b63(pkt,j));
            printf("  MPEG_2_flag                = %x\n",b2(pkt,j));
            printf("  constrained_parameter_flag = %x\n",b1(pkt,j));
            printf("  still_picture_flag         = %x\n",b0(pkt,j));
            if (b2(pkt, j++)) {
                printf("  profile_and_level_indication = %02x\n",b70(pkt, j++));
                printf("  chroma_format                = %x\n",b76(pkt,j));
                printf("  frame_rate_extension_flag    = %x\n",b5(pkt,j));
                printf("  reserved                     = %02x\n",b40(pkt, j++));
            }
            break;

        case AUDIO_STREAM_DESCRIPTOR:                   // 0x03
            printf("  AUDIO DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  free_format_flag           = %x\n",b7(pkt,j));
            printf("  ID                         = %x\n",b6(pkt,j));
            printf("  layer                      = %x\n",b54(pkt,j));
            printf("  reserved                   = %x\n",b30(pkt,j)); j++;
            break;

        case HIERARCHY_DESCRIPTOR:                      // 0x04
            printf("  HIERARCHY DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b74(pkt,j));
            printf("  hierarchy_type             = %x\n",b30(pkt,j)); j++;
            printf("  reserved                   = %x\n",b76(pkt,j));
            printf("  hierarchy_layer_index      = %x\n",b50(pkt,j)); j++;
            printf("  reserved                   = %x\n",b76(pkt,j));
            printf("  hierarchy_embedded_layer   = %x\n",b50(pkt,j)); j++;
            printf("  reserved                   = %x\n",b76(pkt,j));
            printf("  hierarchy_priority         = %x\n",b70(pkt,j)); j++;
            break;

        case REGISTRATION_DESCRIPTOR:                   // 0x05
            printf("  REGISTRATION DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  format_identifier          = %08x\n",b310(pkt,j)); j+=4;
            printf("  additional_id_info         = ");
            while (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j++]);
            }
            printf("\n");
            break;

        case DATA_STREAM_ALIGNMENT_DESCRIPTOR:          // 0x06
            printf("  DATA STREAM ALIGNMENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  alignment_type             = %x\n",pkt[j]); j++;
            break;

        case TARGET_BACKGROUND_GRID_DESCRIPTOR:         // 0x07
            printf("  TARGET BACKGROUND GRID DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  horizontal_size            = %d\n",b152(pkt,j)); j+=2;
            printf("  vertical_size              = %d\n",b174(pkt,j)); j+=2;
            printf("  pel_aspect_ratio           = %x\n",b30(pkt,j)); j++;
            break;

        case VIDEO_WINDOW_DESCRIPTOR:                   // 0x08
            printf("  VIDEO WINDOW DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  horizontal_offset          = %d\n",b152(pkt,j)); j+=2;
            printf("  vertical_offset            = %d\n",b174(pkt,j)); j+=2;
            printf("  window_priority            = %x\n",b30(pkt,j)); j++;
            break;

        case CA_DESCRIPTOR:                             // 0x09
            printf("  CA DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            ca_system_id = b150(pkt,j); j+=2;
            printf("  CA_system_ID               = %04x %s\n",ca_system_id,dvb_enumtype(ca_system_id_str,ca_system_id & 0xff00));
            printf("  reserved                   = %x\n",b75(pkt,j));
            ca_pid = b120(pkt,j); j+=2;
            printf("  CA_PID                     = %04x\n",ca_pid);
            printf("  private_data_byte          = ");
            while (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j]); j++;
            }
            printf("\n");
            break;

        case ISO_639_LANGUAGE_DESCRIPTOR:               // 0x0a
            // may be more than 1 language
            // may not be audio ...
            switch(stream_type) {
                case 0x01:
                case 0x02:
                    //video language
                    break;
                case 0x03:
                case 0x04:
                    //audio language
                    break;
                default:
                    //other language
                    break;
            }

            printf("  ISO_639 LANGUAGE DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            while (j < (descriptor_length+2-1)) {
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[j+0],pkt[j+1],pkt[j+2]); j+=3;
            }
            printf("  audio_type                 = %x\n",pkt[j++]);
            break;

        case SYSTEM_CLOCK_DESCRIPTOR:                   // 0x0b
            printf("  SYSTEM CLOCK DESCRIPTOR\n");
            printf("  descriptor_tag                     = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length                  = %x\n",descriptor_length);
            printf("  external_clock_reference_indicator = %x\n",b7(pkt,j));
            printf("  reserved                           = %x\n",b6(pkt,j));
            printf("  clock_accuracy_integer             = %x\n",b50(pkt,j)); j++;
            printf("  clock_accuracy_exponent            = %x\n",b75(pkt,j));
            printf("  reserved                           = %x\n",b40(pkt,j)); j++;
            break;

        case MULTIPLEX_BUFFER_UTILISATION_DESCRIPTOR:   // 0x0c
            printf("  MULTIPLEX BUFFER UTILISATION DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  mdv_valid_flag             = %x\n",b7(pkt,j));
            printf("  multiplex_delay_variation  = %x\n",b140(pkt,j)); j+=2;
            printf("  multiplex_strategy         = %x\n",b75(pkt,j));
            printf("  reserved                   = %x\n",b40(pkt,j)); j++;
            break;

        case COPYRIGHT_DESCRIPTOR:                      // 0x0d
            printf("  COPYRIGHT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  copyright_identifier       = %08x\n",b310(pkt,j)); j+=4;
            printf("  additional_copyright_info  = ");
            while (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j]); j++;
            }
            printf("\n");
            break;

        case MAXIMUM_BITRATE_DESCRIPTOR:                // 0x0e
            printf("  MAXIMUM BITRATE DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b75(pkt,j));
            printf("  maximum_bitrate            = %x\n",b210(pkt,j)); j+=3;
            break;

        case PRIVATE_DATA_INDICATOR_DESCRIPTOR:         // 0x0f
            printf("  PRIVATE DATA INDICATOR DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  private_data_indicator     = (%08x) %c%c%c\n",b310(pkt,j), pkt[j+3],pkt[j+2],pkt[j+1]); j+=4;
            break;

        case SMOOTHING_BUFFER_DESCRIPTOR:               // 0x10
            printf("  SMOOTHING BUFFER DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b75(pkt,j));
            printf("  sb_leak_rate               = (%06x) %d bits/second\n",b210(pkt,j),b210(pkt,j)*400); j+=3;
            printf("  reserved                   = %x\n",b75(pkt,j));
            printf("  sb_size                    = %06x\n",b210(pkt,j)); j+=3;
            break;

        case STD_DESCRIPTOR:                            // 0x11
            printf("  STD DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b71(pkt,j));
            printf("  leak_valid_flag            = %x\n",b0(pkt,j));  j++;
            break;

        case IBP_DESCRIPTOR:                            // 0x12
            printf("  IBP DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  closed_gop_flag            = %x\n",b7(pkt,j));
            printf("  identical_gop_flag         = %x\n",b6(pkt,j));
            printf("  max_gop-length             = %x\n",b130(pkt,j)); j++;
            break;

        case CAROUSEL_IDENTIFIER_DESCRIPTOR:            // 0x13
            printf("  CAROUSEL IDENTIFIER DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  carousel_id                = %08x\n",b310(pkt,j)); j+=4;
            printf("  format_id                  = %02x\n",b70(pkt,j));
            if (pkt[j] == 0) {
                j++;
                printf("  private_data_byte          = ");
                while (j<(descriptor_length+2)) {
                    printf("%02x ",pkt[j]); j++;
                }
            }
            else if (pkt[j] == 1) {
                j++;
                printf("  module_version             = %02x\n",b70(pkt,j)); j++;
                printf("  module_id                  = %02x\n",b150(pkt,j)); j+=2;
                printf("  block_size                 = %02x\n",b150(pkt,j)); j+=2;
                printf("  module_size                = %02x\n",b310(pkt,j)); j+=4;
                printf("  compression_method         = %02x\n",b70(pkt,j)); j++;
                printf("  original_size              = %02x\n",b310(pkt,j)); j+=4;
                printf("  timeout                    = %02x\n",b70(pkt,j)); j++;
                printf("  object_key_data_length     = %02x\n",b70(pkt,j));
                printf("  object_key_data            = ");
                k = j + b70(pkt,j);
                while (j<k) {
                    printf("%02x ",b70(pkt,1+j)); j++;
                }
                printf("\n");
                j++;
                printf("  private_data_byte          = ");
                while (j<(descriptor_length+2)) {
                    printf("%02x ",b70(pkt,j)); j++;
                }
            }
            else {
                j++;
            }
            printf("\n");
            break;

        case ASSOCIATION_TAG_DESCRIPTOR:                // 0x14     [en301-192]
            printf("  ASSOCIATION TAG DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  association_tag            = %x\n",b150(pkt,j)); j+=2;
            printf("  use                        = %x\n",b150(pkt,j));
            if (b150(pkt,j) == 0) {
                j+=2;
                printf("  selector_length            = %x",pkt[j]); j++;
                printf("  transaction_id             = %x\n",b310(pkt,j)); j+=4;
                printf("  timeout                    = %x\n",b310(pkt,j)); j+=4;
            }
            else if (b150(pkt,j) == 1) {
                j+=2;
                printf("\n  selector_length            = %x",pkt[j]); j++;
            }
            else {
                j+=2;
                printf("  selector_length            = %x",pkt[j]);
                printf("  selector_byte              = ");
                for (i=0; i<pkt[j]; i++) {
                    printf("%02x ",pkt[j+1]); j++;
                }
                j++;
            }
            printf("  private_data               = ");
            while (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j]); j++;
            }
            printf("\n");
            break;

        case DEFERRED_ASSOCIATION_TAGS_DESCRIPTOR:      // 0x15     [en301-192]
            printf("  DEFERRED ASSOCIATION TAGS DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  association_tags_loop_len  = %x",pkt[j]);
            for (i=0; i<pkt[j]; i++) {
                printf("  association_tag            = %x\n",b150(pkt,1+j)); j+=2;
            }
            j++;
            printf("  transport_stream_id        = %x\n",b150(pkt,j)); j+=2;
            printf("  program_nmber              = %x\n",b150(pkt,j)); j+=2;
            printf("  org_network_id             = %x\n",b150(pkt,j)); j+=2;
            printf("  private_data_length        = %x",pkt[j]);
            printf("  private_data               = ");
            for (i=0; i<pkt[j]; i++) {
                printf("%02x ",pkt[j+1]); j++;
            }
            j++;
            printf("\n");
            break;

        case NETWORK_NAME_DESCRIPTOR:                   // 0x40
            to_name(name, (char *)&pkt[j], descriptor_length);
            j += descriptor_length;
            printf("  NETWORK NAME DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  name                       = %s\n",name);
            break;

        case SERVICE_LIST_DESCRIPTOR:                   // 0x41
            printf("  SERVICE LIST DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            while (j<(descriptor_length+2)) {
                printf("  service_id                 = %04x\n",b150(pkt,j)); j+=2;
                printf("  service_type               = (%x) %s\n",pkt[j],dvb_enumtype(service_type_str,pkt[j])); j++;
            }
            break;

        case STUFFING_DESCRIPTOR:                       // 0x42
            printf("  STUFFING DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  data                       = ");
            while (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j]); j++;
            }
            printf("\n");
            break;
#if 0
        case SATELLITE_DELIVERY_SYSTEM_DESCRIPTOR:      // 0x43
            frequency       = to_bcd(b310(pkt,j)); j+=4;
            position        = to_bcd(b150(pkt[6],pkt[7])); j+=2;
            west_east_flag  = b7(pkt[8]);
            polarisation    = b65(pkt[8]);
            modulation      = b40(pkt[8]); j++;
            symbol_rate     = to_bcd(b314(pkt[9],pkt[10],pkt[11],pkt[12])); j+=3;
            fec             = b30(pkt[12]); j++;
            printf("  SATELLITE DELIVERY SYSTEM DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  frequency                  = %d\n",frequency*10);
            printf("  orbital_position           = (%d) %.1f\n",position, position/10.0);
            printf("  west_east_flag             = (%x) %c\n",west_east_flag,(west_east_flag) ? 'E' : 'W');
            printf("  polarisation               = (%x) %s\n",polarisation,dvb_enumtype(polarisation_str,polarisation));
            printf("  modulation                 = (%x) %s\n",modulation,dvb_enumtype(modulation_str,modulation));
            printf("  symbol_rate                = %d\n",symbol_rate*100);
            printf("  FEC_inner                  = (%x) %s\n",fec,dvb_enumtype(fec_str,fec));
            break;

        case BOUQUET_NAME_DESCRIPTOR:                   // 0x47
            to_name(name, (char *)&pkt[2], pkt[1]);
            printf("  BOUQUET NAME DESCRIPTOR\n");
            printf("  descriptor_tag               = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length            = %x\n",descriptor_length);
            printf("  name                         = %s\n",name);
            break;
#endif

        case SERVICE_DESCRIPTOR:                        // 0x48
            service_type = pkt[j++];
            printf("  SERVICE DESCRIPTOR\n");
            printf("  descriptor_tag               = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length            = %x\n",descriptor_length);
            printf("  service_type                 = (%x) %s\n",service_type,dvb_enumtype(service_type_str,service_type));
            to_name(name, (char *)&pkt[j+1], pkt[j+0]);
            if ((t >= 0) && (s >= 0)) strncpy(transponder[t].service[s].provider_name, name, 0x40);
            printf("  service_provider_name_length = %x\n",pkt[j+0]);
            printf("  service_provider_name        = %s\n",name);
            j+=pkt[j+0];
            j++;
            to_name(name, (char *)&pkt[j+1], pkt[j+0]);
            if ((t >= 0) && (s >= 0)) strncpy(transponder[t].service[s].service_name, name, 0x40);
            printf("  service_name_length          = %x\n",pkt[j+0]);
            printf("  service_name                 = %s\n",name);
            j+=pkt[j+0];
            j++;
            break;

#if 0
        case COUNTRY_AVAILABILITY_DESCRIPTOR:           // 0x49
            printf("  COUNTRY AVAILABILITY DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  country_availability_flag  = %x\n",b7(pkt,j));
            printf("  reserved                   = %x\n",b60(pkt,j)); j++;
            while (j < (descriptor_length+2)) {
                printf("  country_code               = %c%c%c\n",pkt[j+0],pkt[j+1],pkt[j+2]); j+=3;
            }
            printf("\n");
            break;

        case LINKAGE_DESCRIPTOR:                        // 0x4a
            printf("  LINKAGE DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  transport_stream_id        = %04x\n",b150(pkt[2],pkt[3])); j+=2;
            printf("  original_network_id        = (%04x) %s\n",b150(pkt[4],pkt[5]), dvb_enumtype(network_id_str,b150(pkt[4],pkt[5]))); j+=2;
            printf("  service_id                 = %04x\n",b150(pkt[6],pkt[7])); j+=2;
            printf("  linkage_type               = (%02x) %s\n",pkt[8],dvb_enumtype(linkage_type_str,pkt[8])); j++;
            printf("  private_data_byte          = ");
            whil (j<(descriptor_length+2)) {
                printf("%02x ",pkt[j]); j++
            }
            printf("\n");
            break;

        case NVOD_REFERENCE_DESCRIPTOR:                 // 0x4b
            printf("  NVOD REFERENCE DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            for (i=2; i<(descriptor_length+2); i+=3) {
                printf("  transport_stream_id        = %04x\n",b150(pkt[i+0],pkt[i+1]));
                printf("  original_network_id        = (%04x) %s\n",b150(pkt[i+2],pkt[i+3]), dvb_enumtype(network_id_str,b150(pkt[i+2],pkt[i+3])));
                printf("  service_id                 = %04x\n",b150(pkt[i+4],pkt[i+5]));
            }
           break;

        case TIME_SHIFTED_SERVICE_DESCRIPTOR:           // 0x4c
            printf("  TIME SHIFTED SERVICE DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reference_service_id       = %04x\n",b150(pkt[2],pkt[3])); j+=2;
            break;

        case SHORT_EVENT_DESCRIPTOR:                    // 0x4d
            to_name(name, (char *)&pkt[6], pkt[5]);
            to_name(text, (char *)&pkt[7+pkt[5]], pkt[6+pkt[5]]);
            printf("  SHORT EVENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  ISO_639_language_code      = %c%c%c",pkt[2],pkt[3],pkt[4]); j+=3;
            printf("  event_name_length          = %02x",pkt[5]); j++
            printf("  event_name                 = %s",name);
            printf("  text_name_length           = %02x",pkt[6+pkt[5]]);
            printf("  text                       = %s",text);
            break;

        case EXTENDED_EVENT_DESCRIPTOR:                 // 0x4e
            printf("  EXTENDED EVENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  descriptor_number          = %x\n",b74(pkt[2]));
            printf("  last_descriptor_number     = %x\n",b30(pkt[2])); j++;
            printf("  ISO_639_language_code      = %c%c%c\n",pkt[3],pkt[4],pkt[5]); j+=3;
            printf("  length_of_items            = %x\n",pkt[6]); j++;
            j = 0;
            while (j<pkt[6]) {
                to_name(name, (char *)&pkt[8+j], pkt[7+j]);
                printf("  item_description_length    = %x\n",pkt[7+j]);
                printf("  item_description           = %s\n",name);
                j = j + 1 + pkt[7+j];

                to_name(text, (char *)&pkt[8+j], pkt[7+j]);
                printf("  item_length                = %x\n",pkt[7+j]);
                printf("  item                       = %s\n",text);
                j = j + 1 + pkt[7+j];
            }
            to_name(text, (char *)&pkt[8+pkt[6]], pkt[7+pkt[6]]);
            printf("  text_length                = %x\n",pkt[7+j]);
            printf("  text                       = %s\n",text);
            break;

        case TIME_SHIFTED_EVENT_DESCRIPTOR:             // 0x4f
            printf("  TIME SHIFTED EVENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reference_service_id       = %04x\n",b150(pkt[2],pkt[3])); j+=2;
            printf("  reference_event_id         = %04x\n",b150(pkt[4],pkt[5])); j+=2;
            break;
#endif

        case COMPONENT_DESCRIPTOR:                      // 0x50
            printf("  COMPONENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved_future_use        = %x\n",b74(pkt,j));
            printf("  stream_content             = (%x %02x) %s\n",b30(pkt,j),b70(pkt,j+1),get_component_type_str(b30(pkt,j),pkt[j+1])); j++;
            printf("  component_type             = %x\n",pkt[j]); j++;
            printf("  component_tag              = %x\n",pkt[j]); j++;
            printf("  ISO_639_language_code      = %c%c%c\n",pkt[j+0],pkt[j+1],pkt[j+2]); j+=3;
            to_name(text, (char *)&pkt[j], descriptor_length+2-8); j+=(descriptor_length+2-8);
            printf("  text                       = %s\n",text);
            break;

        case STREAM_IDENTIFIER_DESCRIPTOR:              // 0x52
            printf("  STREAM IDENTIFIER DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  component_tag              = %x\n",pkt[j]); j++;
            break;

        case CA_IDENTIFIER_DESCRIPTOR:                  // 0x53
            printf("  CA IDENTIFIER DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
           while (j<(descriptor_length+2)) {
                printf("  CA_system_id               = (%04x) %s\n",b150(pkt,j),dvb_enumtype(ca_system_id_str,b150(pkt,j))); j+=2;
            }
            break;

        case CONTENT_DESCRIPTOR:                        // 0x54
            printf("  CONTENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
           while (j<(descriptor_length+2)) {
                printf("  content_nibble_level_1     = (%x) %s\n",b74(pkt,j),get_content_type_str(b74(pkt,j+1), b30(pkt,j))); j++;
                printf("  content_nibble_level_2     = %x\n",b30(pkt,j)); j++;
                printf("  user_nibble1               = %x\n",b74(pkt,j));
                printf("  user_nibble2               = %x\n",b30(pkt,j)); j++;
            }
            break;

        case PARENTAL_RATING_DESCRIPTOR:                // 0x55
            printf("  PARENTAL RATING DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
           while (j<(descriptor_length+2)) {
                printf("  country_code               = %c%c%c\n",pkt[0+j],pkt[1+j],pkt[2+j]); j+=3;
                printf("  rating                     = (%x) CERT %d\n", pkt[j], (pkt[j]) ? pkt[j]+3 : 0); j++;
            }
            break;

        case TELETEXT_DESCRIPTOR:                       // 0x56
            printf("  TELETEXT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
           while (j<(descriptor_length+2)) {
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[0+j],pkt[1+j],pkt[2+j]); j+=3;
                printf("  teletext_type              = %x\n",pkt[j]); j++;
                printf("  teletext_magazine_number   = %x\n",pkt[j]); j++;
                printf("  teletext_page_number       = %x\n",pkt[j]); j++;
            }
            break;

        case LOCAL_TIME_OFFSET_DESCRIPTOR:              // 0x58
            printf("  LOCAL TIME OFFSET DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
           while (j<(descriptor_length+2)) {
                printf("  country_code               = %c%c%c\n", pkt[0+j],pkt[1+j],pkt[2+j]); j+=3;
                printf("  country_region_id          = %x\n",b72(pkt,j));
                printf("  reserved                   = %x\n",b1(pkt,j));
                printf("  local_time_offset_polarity = %x\n",b0(pkt,j)); j++;
                printf("  local_time_offset          = %x\n",b150(pkt,j)); j+=2;
                printf("  time_of_change             = %s\n",getmjd(&pkt[j])); j+=5;
                printf("  next_time_offset           = %x\n",b150(pkt,j)); j+=2;
            }
            break;

        case SUBTITLING_DESCRIPTOR:                     // 0x59
            printf("  SUBTITLING DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            while (j<(descriptor_length+2)) {
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[0+j],pkt[1+j],pkt[2+j]); j+=3;
                printf("  subtitling_type            = %x\n",pkt[j]); j++;
                printf("  composition_page_id        = %x\n",b150(pkt,j)); j+=2;
                printf("  ancillary_page_id          = %x\n",b150(pkt,j)); j+=2;
            }
            break;

        case TERRESTRIAL_DELIVERY_SYSTEM_DESCRIPTOR:    // 0x5a
            printf("  TERRESTRIAL DELIVERY SYSTEM DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            frequency = b310(pkt,j); j+=4;
            printf("  centre_frequency           = (%08x) %.3fMHz\n",frequency,(float)(frequency*10/1000000.0));
            bandwidth = b75(pkt,j);
            printf("  bandwidth                  = (%x) %s\n",bandwidth,dvb_enumtype(bandwidth_str,bandwidth));
//          printf("  reserved                   = %x\n",b40(pkt,j)); j++;
            modulation = b76(pkt,j);
            printf("  constellation              = (%x) %s\n",modulation,dvb_enumtype(constellation_str,modulation));
            hierarchy = b53(pkt,j);
            printf("  hierarchy_information      = (%x) %s\n",hierarchy,dvb_enumtype(hierarchy_str,hierarchy));
            fec_high = b20(pkt,j);
            printf("  code_rate_HP_stream        = (%x) %s\n",fec_high,dvb_enumtype(code_rate_str,fec_high)); j++;
            fec_low = b75(pkt,j);
            printf("  code_rate_LP_stream        = (%x) %s\n",fec_low,dvb_enumtype(code_rate_str,fec_low));
            guard_interval = b43(pkt,j);
            printf("  guard_interval             = (%x) %s\n",guard_interval,dvb_enumtype(guard_interval_str,guard_interval));
            transmission_mode = b21(pkt,j);
            printf("  transmission_mode          = (%x) %s\n",transmission_mode,dvb_enumtype(transmission_mode_str,transmission_mode));
//          printf("  other_frequency_flag       = %x\n",b0(pkt,j)); j++;
//          printf("  reserved                   = %x\n",b310(pkt,j)); j+=4;
            break;

#if 0
        case MULTILINGUAL_NETWORK_NAME_DESCRIPTOR:      // 0x5b
            printf("  MULTILINGUAL NETWORK NAME DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            j = 0;
            while(j<(descriptor_length)) {
                to_name(name, (char *)&pkt[6+j], pkt[5+j]);
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[2+j],pkt[3+j],pkt[4+j]);
                printf("  network_name_length        = %x\n",pkt[5+j]);
                printf("  network_name               = %s\n",name);
                j+=4+pkt[5+j];
            }
            break;

        case MULTILINGUAL_BOUQUET_NAME_DESCRIPTOR:      // 0x5c
            printf("  MULTILINGUAL BOUQUET NAME DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            j = 0;
            while(j<(descriptor_length)) {
                to_name(name, (char *)&pkt[6+j], pkt[5+j]);
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[2+j],pkt[3+j],pkt[4+j]);
                printf("  bouquet_name_length        = %x\n",pkt[5+j]);
                printf("  bouquet_name               = %s\n",name);
                j+=4+pkt[5+j];
            }
            break;

        case MULTILINGUAL_SERVICE_NAME_DESCRIPTOR:      // 0x5d
            printf("  MULTILINGUAL SERVICE NAME DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            j = 0;
            while(j<(descriptor_length)) {
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[2+j],pkt[3+j],pkt[4+j]);
                to_name(name, (char *)&pkt[6+j], pkt[5+j]);
                printf("  provider_name_length       = %x\n",pkt[5+j]);
                printf("  provider_name              = %s\n",name);
                to_name(name, (char *)&pkt[7+j+pkt[6+j+pkt[5+j]]], pkt[6+j+pkt[5+j]]);
                printf("  service_name_length        = %x\n",pkt[6+j+pkt[5+j]]);
                printf("  service_name               = %s\n",name);
                j+=4+pkt[5+j]+1+pkt[6+j+pkt[5+j]];
            }
            break;

        case MULTILINGUAL_COMPONENT_DESCRIPTOR:         // 0x5e
            printf("  MULTILINGUAL COMPONENT DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  component_tag              = %x\n",pkt[j]); j++;
            j = 0;
            while(j<(descriptor_length-1)) {
                to_name(name, (char *)&pkt[7+j], pkt[6+j]);
                printf("  ISO_639_language_code      = %c%c%c\n",pkt[0+j],pkt[1+j],pkt[2+j]); j+=3;
                printf("  text_description_length    = %x\n",pkt[j]); j++;
                printf("  component_name             = %s\n",name);
                j+=4+pkt[6+j];
            }
            break;
#endif

        case PRIVATE_DATA_SPECIFIER_DESCRIPTOR:         // 0x5f
            printf("  PRIVATE DATA SPECIFIER DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  private_data_specifier     = (%08x) [%c%c%c] %s\n",b310(pkt,j), pkt[j+3],pkt[j+2],pkt[j+1], dvb_enumtype(private_data_specifier_str,b310(pkt,j)));  j+=4;
            break;

        case FREQUENCY_LIST_DESCRIPTOR:                 // 0x62
            printf("  FREQUENCY LIST DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b72(pkt,j));
            printf("  coding_type                = (%x) %s\n",b10(pkt,j),dvb_enumtype(coding_type_str,b10(pkt,j))); j++;
            while (j<(descriptor_length+2)) {
                frequency = b310(pkt,j); j+=4;
                printf("  centre_frequency           = (%08x) %.3fMHz\n",frequency,(float)(frequency*10/1000000.0));
            }
            break;

        case DATA_BROADCAST_DESCRIPTOR:                 // 0x64
            printf("  DATA BROADCAST DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  data_broadcast_id          = (%04x) %s\n",b150(pkt,j),dvb_enumtype(data_broadcast_id_str,pkt[j+1]));   j+=2;
            printf("  component_tag              = %02x\n",b70(pkt,j)); j++;
            printf("  selector_length            = %02x\n",b70(pkt,j)); j++;
            if (pkt[3] == 0x06) {
                printf("  carousel_type_id           = (%02x) %s\n",b76(pkt,j),dvb_enumtype(carousel_type_id_str,b76(pkt,j)));
                printf("  reserved                   = %x\n",b50(pkt,j)); j++;
                printf("  transaction_id             = %08x\n",b310(pkt,j)); j+=4;
                printf("  time_out_value_DSI         = %08x\n",b310(pkt,j)); j+=4;
                printf("  time_out_value_DII         = %08x\n",b310(pkt,j)); j+=4;
                printf("  reserved                   = %x\n",b76(pkt,j));
                printf("  leak_rate                  = %06x\n",b210(pkt,j)); j+=3;
            }
            else {
                printf("  selector_bytes             = ");
                for (i=0; i<pkt[5]; i++) {
                    printf( "%02x ",pkt[j]); j++;
                }
                printf("\n");
            }
/*
[see en301-192]
data_broadcast_id == 1 => dvb data pipe
data_broadcast_id == 2 => asynchronous data stream
data_broadcast_id == 4 => synchronous data stream      - pes
data_broadcast_id == 5 => multiprotocol encapsulation  => 0x3E
data_broadcast_id == 6 => DVB data carousel
data_broadcast_id == 7 => DVB object carousel
*/
            printf("  language_code              = %c%c%c\n",pkt[j+0],pkt[j+1],pkt[j+2]); j+=3;
            to_name(text, (char *)&pkt[j+1], pkt[j]);
            printf("  text_length                = %02x\n",pkt[j]); j+=pkt[j]; j++;
            printf("  text                       = %s\n",text);
            break;

        case DATA_BROADCAST_ID_DESCRIPTOR:              // 0x66
            printf("  DATA BROADCAST ID DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  data_broadcast_id          = %04x\n",b150(pkt,j)); j+=2;
//            printf("  id_selector                = ");
//            for (i=4; i<(descriptor_length+2); i++) {
//                printf( "%02x ",pkt[i]);
//            }
//            printf("\n");

            if (descriptor_length == 4) {
                printf("  application_type_code      = %04x\n",b150(pkt,j)); j+=2;
            }
            else {
                while(j<(descriptor_length-2)) {
                    printf("  application_type_code      = %04x\n",b150(pkt,j)); j+=2;
                    printf("  boot_priority_hint         = %02x\n",pkt[j++]);
                    printf("  application_specfic_length = %02x\n",pkt[j]);
                    printf("  application_specfic_data   = ");
                    if (pkt[j]) {
                        for (i=0; i<pkt[j]; i++) {
                            printf("%02x ",pkt[j+1+i]);
                        }
                        printf("\n");
                        to_name(text, (char *)&pkt[j+1], pkt[j]);
                        printf("  application_specfic_data   = %s\n",text);
                    }
                    j += pkt[j];
                    j++;
                }
            }
            break;

        case PDC_DESCRIPTOR:                            // 0x69
            printf("  PDC DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  reserved                   = %x\n",b74(pkt,j));
            program_id_label = b190(pkt,j); j+=3;
            printf("  program_id_label           = (%05x) %02d/%02d %02d:%02d",program_id_label,
                (program_id_label >> 15) & 0x1f, (program_id_label >> 11) & 0xf, (program_id_label >> 6) & 0x1f, (program_id_label >> 0) & 0x3f );
            break;

        case ANCILLARY_DESCRIPTOR:                      // 0x6b
            printf("  ANCILLARY DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            printf("  ancillary_data_identifier  = (%02x) ",pkt[j]);
            if (pkt[j] & 0x01) printf("DVD-Video ");
            if (pkt[j] & 0x02) printf("Extended ");
            if (pkt[j] & 0x04) printf("Announcement Switching Data ");
            if (pkt[j] & 0x08) printf("DAB ");
            if (pkt[j] & 0x10) printf("Scale Factor Error Check ");
            if (pkt[j] & 0x20) printf("rsvd[5] ");
            if (pkt[j] & 0x40) printf("rsvd[6] ");
            if (pkt[j] & 0x80) printf("rsvd[7] ");
            printf("\n");
            j++;
            break;


        case CHANNEL_MAP_DESCRIPTOR:                    // 0x83
            printf("  CHANNEL MAP DESCRIPTOR\n");
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_length          = %x\n",descriptor_length);
            while (j<(descriptor_length+2)) {
                printf("  service_id                 = %04x\n",b150(pkt,j)); j+=2;
//              printf("  reserved                   = %x\n",b72(pkt,j));
                printf("  channel                    = %d\n",b90(pkt,j)); j+=2;
            }
            break;








        case CABLE_DELIVERY_SYSTEM_DESCRIPTOR:          // 0x44
        case MOSAIC_DESCRIPTOR:                         // 0x51
        case TELEPHONE_DESCRIPTOR:                      // 0x57
        case SERVICE_MOVE_DESCRIPTOR:                   // 0x60
        case SHORT_SMOOTHING_BUFFER_DESCRIPTOR:         // 0x61
        case PARTIAL_TRANSPORT_STREAM_DESCRIPTOR:       // 0x63
        case CA_SYSTEM_DESCRIPTOR:                      // 0x65
        case TRANSPORT_STREAM_DESCRIPTOR:               // 0x67
        case DSNG_DESCRIPTOR:                           // 0x68
        case AC3_DESCRIPTOR:                            // 0x6a
        case CELL_LIST_DESCRIPTOR:                      // 0x6c
        case CELL_FREQUENCY_LINK_DESCRIPTOR:            // 0x6d
        case ANNOUNCEMENT_SUPPORT_DESCRIPTOR:           // 0x6e
//            break;

        default:
            printf("  UNKNOWN DESCRIPTOR\n");
//            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,dvb_enumtype(descriptor_str,descriptor_tag));
            printf("  descriptor_tag             = (%02x) %s\n",descriptor_tag,"");
            printf("  descriptor_length          = %x\n",descriptor_length);
            for (i=0; i<descriptor_length; i+=0x10) {
                printf( "   %04x : ",i);
                for (j=0; j<0x10; j++) {
                    if ((i+j) < descriptor_length) {
                        printf( "%02x ",pkt[2+i+j]);
                    }
                    else {
                        printf( "   ");
                    }
                }
                to_name(text, (char *)&pkt[2+i], (descriptor_length<=0x10)?descriptor_length:0x10);
                printf( "%s\n",text);
            }
            break;
    }
    return 2+descriptor_length;
}


int dump_pat(unsigned char *buf)
{
    int service_id;
    int section_length, num_services;
    int i,j;

    j = 0;
    printf("\n");
    printf("PROGRAM ASSOCIATION SECTION\n");
    display_section_header(buf, "transport_stream_id");
    j++;
    section_length = 3 + b110(buf,j); j+=2;
    j += 3;

    num_services = (section_length-8-4)/4;
    for (i=0; i<num_services; i++) {
        service_id = b150(buf,j); j+=2;
        printf("service_id                   = %04x\n",service_id);
        if (service_id != 0) {
            printf("program_map_pid              = %04x\n",b120(buf,j));
        } else {
            printf("network_pid                  = %04x\n",b120(buf,j));
        }
        j+=2;
    }

    return num_services;
}


int dump_nit(unsigned char *buf)
{
    int network_id, transport_stream_id, original_network_id;
    int section_length, descriptor_length;
    int j,k,t;

    j = 0;
    printf("\n");
    printf("NETWORK INFORMATION SECTION\n");
    display_section_header(buf, "network_id         ");
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    network_id = b150(buf, j); j+=2;
    j += 3;
    descriptor_length = b110(buf, j); j+=2;
    printf("network_descriptors_length   = %03x\n",descriptor_length);

    // skip the descriptors   - do later ...
    k = j;
    while(j < (k+descriptor_length)) {
//        j += dump_desc(&buf[j],-1,-1,-1);
        j += b70(buf, j+1);
        j += 2;
    }
    printf("transport_stream_loop_length = %03x\n",b110(buf,j)); j+=2;
    while (j < (section_length-4)) {
        transport_stream_id = b150(buf, j); j+= 2;
        t = add_transport_stream_id(transport_stream_id);
        original_network_id = b150(buf, j); j+= 2;
        printf("transport_stream_id          = %04x\n",transport_stream_id);
        printf("original_network_id          = (%04x) %s\n",original_network_id, dvb_enumtype(network_id_str,original_network_id));
        printf("transport_descriptors_length = %x\n",descriptor_length);

        descriptor_length = b110(buf, j); j+= 2;
        k = j;
        while (j < (k+descriptor_length)) {
            j += dump_desc(&buf[j],t,-1,-1);
        }
    }

    // do these now, so that we know the sat
    j = 8;
    descriptor_length = b110(buf, j); j+=2;
    k = j;
    while(j < (k+descriptor_length)) {
        j += dump_desc(&buf[j],t,-1,-1);
    }

    return num_transponders;
}


int dump_sdt(unsigned char *buf)
{
    int transport_stream_id, original_network_id, service_id;
    int section_length, descriptor_length;
    int j,k,t,s;
    int eit_schedule_flag,eit_present_following_flag,running_status,free_ca_mode;

    j = 0;
    printf("\n");
    printf("SERVICE DESCRIPTOR SECTION\n");
    display_section_header(buf, "transport_stream_id");
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    transport_stream_id = b150(buf, j); j+=2;
    t = add_transport_stream_id(transport_stream_id);
    j+=3;
    original_network_id = b150(buf, j); j+=2;
    printf("original_network_id          = (%04x) %s\n",original_network_id, dvb_enumtype(network_id_str,original_network_id));
    j ++;
    while (j<(section_length-4)) {
        service_id                  = b150(buf, j); j+=2;
        s = add_service_id(t, service_id);
        eit_schedule_flag           = b1(buf, j);
        eit_present_following_flag  = b0(buf, j); j++;
        running_status              = b75(buf, j);
        free_ca_mode                = b4(buf, j);
        descriptor_length           = b110(buf, j); j+=2;

        printf("service_id                   = %04x\n",service_id);
        printf("EIT_schedule_flag            = (%x) %s\n",eit_schedule_flag, dvb_enumtype(no_yes_str,eit_schedule_flag));
        printf("EIT_present_following_flag   = (%x) %s\n",eit_present_following_flag, dvb_enumtype(no_yes_str,eit_present_following_flag));
        printf("running_status               = (%x) %s\n",running_status, dvb_enumtype(running_status_str,running_status));
        printf("free_CA_mode                 = (%x) %s\n",free_ca_mode, dvb_enumtype(free_ca_str,free_ca_mode));
        printf("descriptors_loop_length      = %x\n",descriptor_length);

        k = j;
        while (j < (k+descriptor_length)) {
            j += dump_desc(&buf[j],t,s,-1);
        }
    }

    return 0;
}


int dump_pmt(unsigned char *buf, int t)
{
    int service_id, elementary_pid, stream_type, pcr_pid;
    int section_length, descriptor_length;
    int j,k,s;

    j = 0;
    printf("\n");
    printf("TS PROGRAM MAP SECTION\n");
    display_section_header(buf, "service_id         ");
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    service_id = b150(buf, j); j+=2;
    s = add_service_id(t, service_id);
    j += 3;
    pcr_pid = b120(buf, j); j+=2;
    descriptor_length = b110(buf, j); j+=2;
    printf("pcr_pid                      = %04x\n",pcr_pid);
    printf("program_info_length          = %03x\n",descriptor_length);
    k = j;
    while (j < (k+descriptor_length)) {
        j += dump_desc(&buf[j],t,s,-1);
    }

    while (j<(section_length-4)) {
        stream_type           = b70(buf, j++);
        elementary_pid        = b120(buf, j); j+=2;
        printf("stream_type                  = %02x, %s\n",stream_type,dvb_enumtype(stream_type_str,stream_type));
        printf("elementary_PID               = %04x",elementary_pid);
        switch(stream_type) {
        case 0x01:
        case 0x02:
            //video pid
            if (debug) printf(" video");
            break;
        case 0x03:
        case 0x04:
            //audio pid
            if (debug) printf(" audio");
            break;
        case 0x05:
            break;
        case 0x06:
            //pes pid
            break;
        case 0x0a:
        case 0x0b:
        case 0x0c:
        case 0x0d:
        case 0x0e:
            //dsmcc pid
            break;
        case 0xc0: // patch
        case 0xc1: // patch
        case 0xc6: // program_map
        case 0xc8: // download s/w
            //s/w downlod pid
            break;
        default:
            //other data pid
//            lprintf(LOG_EDIT,"\nstream_type %02x - unrecognised, program_number=%04x\n",stream_type,program_number);
            break;
        }
        printf("\n");
        descriptor_length     = b110(buf, j); j+=2;
        printf("ES_info_length               = %03x\n",descriptor_length);
        k = j;
        while (j < (k+descriptor_length)) {
            j += dump_desc(&buf[j],t,s,stream_type);
        }
    }

    return 0;
}

