/*



*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>

#include "dvb.h"
#include "parse_dvb.h"

//----------------------------------------------------------------------------
//#define DESCLEN 0x40
//typedef struct
//{
//    int   id;
//    char  description[DESCLEN];
//} EnumType;


// stream type assignments
EnumType stream_type_str[] = {
    { 0x0e, "UNKNOWN"},
    { 0x01, "11172 VIDEO"},
    { 0x02, "13818 VIDEO"},
    { 0x03, "11172 AUDIO"},
    { 0x04, "13818 AUDIO"},
    { 0x05, "PRIVATE SECTION"},
    { 0x06, "PES PRIVATE SECTION"},
    { 0x07, "MHEG"},
    { 0x08, "DSM CC"},
    { 0x09, "AUX"},
    { 0x0a, "13818-6 TYPE A"},
    { 0x0b, "13818-6 TYPE B"},
    { 0x0c, "13818-6 TYPE C"},
    { 0x0d, "13818-6 TYPE D"},
};

EnumType service_type_str[] = {
    { 0x11, "unknown"              },
    { 0x00, "reserved"             },
    { 0x01, "Digital Television"   },
    { 0x02, "Digital Radio Sound"  },
    { 0x03, "Teletext"             },
    { 0x04, "NVOD Reference"       },
    { 0x05, "NVOD Time-shifted"    },
    { 0x06, "Mosaic"               },
    { 0x07, "PAL coded signal"     },
    { 0x08, "SECAM coded signal"   },
    { 0x09, "D/D2-MAC coded"       },
    { 0x0a, "FM Radio"             },
    { 0x0b, "NTSC coded"           },
    { 0x0c, "Data Broadcast"       },
    { 0x0d, "Common Interface"     },
    { 0x0e, "RCS map"              },
    { 0x0f, "RCS FLS"              },
    { 0x10, "DVB MHP"              },
};

//terrestrial delivery system, bandwidth
EnumType bandwidth_str[] = {
// types
    { 0x02, "reserved"                 },
    { 0x00, "8 MHz"                    },
    { 0x01, "7 MHz"                    }
};

//terrestrial delivery system, constellation
EnumType constellation_str[] = {
// types
    { 0x03, "reserved"                 },
    { 0x00, "QPSK"                     },
    { 0x01, "16-QAM"                   },
    { 0x02, "64-QAM"                   }
};

//terrestrial delivery system, hierarchy
EnumType hierarchy_str[] = {
// types
    { 0x04, "reserved"                 },
    { 0x00, "non-hierarchical"         },
    { 0x01, "alpha=1"                  },
    { 0x02, "alpha=2"                  },
    { 0x03, "alpha=4"                  }
};

//terrestrial delivery system, code rate
EnumType code_rate_str[] = {
// types
    { 0x05, "reserved"                 },
    { 0x00, "1/2"                      },
    { 0x01, "2/3"                      },
    { 0x02, "3/4"                      },
    { 0x03, "5/6"                      },
    { 0x04, "7/8"                      }
};

//terrestrial delivery system, guard interval
EnumType guard_interval_str[] = {
// types
    { 0x04, "reserved"                 },
    { 0x00, "1/32"                     },
    { 0x01, "1/16"                     },
    { 0x02, "1/8"                      },
    { 0x03, "1/4"                      }
};

//terrestrial delivery system, transmission mode
EnumType transmission_mode_str[] = {
// types
    { 0x02, "reserved"                 },
    { 0x00, "2K"                       },
    { 0x01, "8K"                       }
};

//----------------------------------------------------------------------------
void dump(void *buf, int len)
{
    int i,j;

    for (i=0; i<len; i+=0x10) {
        printf("%04x : ",i);
        for (j=0; j<0x10; j++) {
            if ((i+j)<len) printf("%02x ", ((unsigned char *)buf)[i+j]); else printf("   ");
        }
        printf("  ");
        for (j=0; j<0x10; j++) {
            if ((i+j)<len) printf("%c", to_printable(((unsigned char *)buf)[i+j])); else printf(" ");
        }
        printf("\n");
    }
}

//----------------------------------------------------------------------------

unsigned char *dvb_enumtype(EnumType *qenum, int id)
{
    int i;

    for(i=1; i<=qenum[0].id; i++) {
        if (qenum[i].id == id) {
            return qenum[i].description;
        }
    }
    return qenum[0].description;
}


// strip non-ascii chars from string
int to_name(char *dest_str, char *source_str, int len)
{
    int i,j;

    j = 0;
    for (i=0; i<len; i++) {
        if (source_str[i] >= 0x20) {
            dest_str[j++] = source_str[i];
        }
    }
    dest_str[j++] = '\0';
    return 0;
}


int add_transport_stream_id(int transport_stream_id)
{
    int t;

    for (t=0; t<num_transponders; t++) {
        if (transponder[t].transport_stream_id  == transport_stream_id) {
            return t;
        }
    }
    transponder[t].transport_stream_id  = transport_stream_id;
    num_transponders ++;
    return t;
}


int add_service_id(int t, int service_id)
{
    int s;

    for (s=0; s<transponder[t].num_services; s++) {
        if (transponder[t].service[s].service_id  == service_id) return s;
    }
    transponder[t].service[s].service_id  = service_id;
    transponder[t].num_services ++;
    return s;
}


int do_desc(unsigned char *pkt, int t, int s, int stream_type)
{
    int descriptor_tag, descriptor_length;
    int frequency;
    int modulation;
    int bandwidth, guard_interval, transmission_mode, hierarchy, fec_high, fec_low;
    int j;
    int service_id;
    int service_type;
    int lcn;
    char pname[0x102];
    char name[0x102];

    j = 0;
    descriptor_tag = b70(pkt, j++);
    descriptor_length = b70(pkt, j++);
    if (!descriptor_length) return 2;

    switch (descriptor_tag)  {
        case VIDEO_STREAM_DESCRIPTOR:                   // 0x02
        case AUDIO_STREAM_DESCRIPTOR:                   // 0x03
        case HIERARCHY_DESCRIPTOR:                      // 0x04
        case REGISTRATION_DESCRIPTOR:                   // 0x05
        case DATA_STREAM_ALIGNMENT_DESCRIPTOR:          // 0x06
        case TARGET_BACKGROUND_GRID_DESCRIPTOR:         // 0x07
        case VIDEO_WINDOW_DESCRIPTOR:                   // 0x08
        case CA_DESCRIPTOR:                             // 0x09
        case ISO_639_LANGUAGE_DESCRIPTOR:               // 0x0a
        case SYSTEM_CLOCK_DESCRIPTOR:                   // 0x0b
        case MULTIPLEX_BUFFER_UTILISATION_DESCRIPTOR:   // 0x0c
        case COPYRIGHT_DESCRIPTOR:                      // 0x0d
        case MAXIMUM_BITRATE_DESCRIPTOR:                // 0x0e
        case PRIVATE_DATA_INDICATOR_DESCRIPTOR:         // 0x0f
        case SMOOTHING_BUFFER_DESCRIPTOR:               // 0x10
        case STD_DESCRIPTOR:                            // 0x11
        case IBP_DESCRIPTOR:                            // 0x12
        case CAROUSEL_IDENTIFIER_DESCRIPTOR:            // 0x13
        case ASSOCIATION_TAG_DESCRIPTOR:                // 0x14     [en301-192]
        case DEFERRED_ASSOCIATION_TAGS_DESCRIPTOR:      // 0x15     [en301-192]
            break;

        case NETWORK_NAME_DESCRIPTOR:                   // 0x40
            to_name(name, (char *)&pkt[j], descriptor_length);
            j += descriptor_length;
            if (t >= 0) {
                strncpy(transponder[t].network_name, name, 0x40);
            }
            break;

        case SERVICE_LIST_DESCRIPTOR:                   // 0x41
            s = 0;
            while (j<(descriptor_length+2)) {
                service_id = b150(pkt,j);
                j+=2;
                service_type = pkt[j];
                j++;
                s++;
            }
            if ((t >= 0) && (s >= 0)) {
                transponder[t].service[s].service_id = service_id;
                transponder[t].service[s].service_type = service_type;
            }
            break;

        case STUFFING_DESCRIPTOR:                       // 0x42
        case SATELLITE_DELIVERY_SYSTEM_DESCRIPTOR:      // 0x43
        case CABLE_DELIVERY_SYSTEM_DESCRIPTOR:          // 0x44
        case BOUQUET_NAME_DESCRIPTOR:                   // 0x47
            break;

        case SERVICE_DESCRIPTOR:                        // 0x48
            service_type = pkt[j++];
            to_name(pname, (char *)&pkt[j+1], pkt[j+0]);
            j+=pkt[j+0];
            j++;
            to_name(name, (char *)&pkt[j+1], pkt[j+0]);
            j+=pkt[j+0];
            j++;
            if ((t >= 0) && (s >= 0)) {
                transponder[t].service[s].service_type = service_type;
                strncpy(transponder[t].service[s].provider_name, pname, 0x40);
                strncpy(transponder[t].service[s].service_name, name, 0x40);
            }
            break;

        case COUNTRY_AVAILABILITY_DESCRIPTOR:           // 0x49
        case LINKAGE_DESCRIPTOR:                        // 0x4a
        case NVOD_REFERENCE_DESCRIPTOR:                 // 0x4b
        case TIME_SHIFTED_SERVICE_DESCRIPTOR:           // 0x4c
        case SHORT_EVENT_DESCRIPTOR:                    // 0x4d
        case EXTENDED_EVENT_DESCRIPTOR:                 // 0x4e
        case TIME_SHIFTED_EVENT_DESCRIPTOR:             // 0x4f
        case COMPONENT_DESCRIPTOR:                      // 0x50
        case MOSAIC_DESCRIPTOR:                         // 0x51
        case STREAM_IDENTIFIER_DESCRIPTOR:              // 0x52
        case CA_IDENTIFIER_DESCRIPTOR:                  // 0x53
        case CONTENT_DESCRIPTOR:                        // 0x54
        case PARENTAL_RATING_DESCRIPTOR:                // 0x55
        case TELETEXT_DESCRIPTOR:                       // 0x56
        case TELEPHONE_DESCRIPTOR:                      // 0x57
        case LOCAL_TIME_OFFSET_DESCRIPTOR:              // 0x58
        case SUBTITLING_DESCRIPTOR:                     // 0x59
            break;

        case TERRESTRIAL_DELIVERY_SYSTEM_DESCRIPTOR:    // 0x5a
            frequency = b310(pkt,j); j+=4;
            bandwidth = b75(pkt,j);
            j++;
            modulation = b76(pkt,j);
            hierarchy = b53(pkt,j);
            fec_high = b20(pkt,j); j++;
            fec_low = b75(pkt,j);
            guard_interval = b43(pkt,j);
            transmission_mode = b21(pkt,j);
            j++;
            j+=4;
            if (t >= 0) {
                transponder[t].frequency = frequency*10;
                transponder[t].bandwidth = bandwidth;
                transponder[t].modulation = modulation;
                transponder[t].hierarchy = hierarchy;
                transponder[t].fec_high = fec_high;
                transponder[t].fec_low = fec_low;
                transponder[t].guard_interval = guard_interval;
                transponder[t].transmission_mode = transmission_mode;
            }
            break;

        case MULTILINGUAL_NETWORK_NAME_DESCRIPTOR:      // 0x5b
        case MULTILINGUAL_BOUQUET_NAME_DESCRIPTOR:      // 0x5c
        case MULTILINGUAL_SERVICE_NAME_DESCRIPTOR:      // 0x5d
        case MULTILINGUAL_COMPONENT_DESCRIPTOR:         // 0x5e
        case PRIVATE_DATA_SPECIFIER_DESCRIPTOR:         // 0x5f
        case SERVICE_MOVE_DESCRIPTOR:                   // 0x60
        case SHORT_SMOOTHING_BUFFER_DESCRIPTOR:         // 0x61
        case FREQUENCY_LIST_DESCRIPTOR:                 // 0x62
        case PARTIAL_TRANSPORT_STREAM_DESCRIPTOR:       // 0x63
        case DATA_BROADCAST_DESCRIPTOR:                 // 0x64
        case CA_SYSTEM_DESCRIPTOR:                      // 0x65
        case DATA_BROADCAST_ID_DESCRIPTOR:              // 0x66
        case TRANSPORT_STREAM_DESCRIPTOR:               // 0x67
        case DSNG_DESCRIPTOR:                           // 0x68
        case PDC_DESCRIPTOR:                            // 0x69
        case AC3_DESCRIPTOR:                            // 0x6a
        case ANCILLARY_DESCRIPTOR:                      // 0x6b
        case CELL_LIST_DESCRIPTOR:                      // 0x6c
        case CELL_FREQUENCY_LINK_DESCRIPTOR:            // 0x6d
        case ANNOUNCEMENT_SUPPORT_DESCRIPTOR:           // 0x6e
            break;

        case CHANNEL_MAP_DESCRIPTOR:                    // 0x83
            while (j<(descriptor_length+2)) {
                service_id = b150(pkt,j);
                s = add_service_id(t, service_id);
                j+=2;
                lcn = b90(pkt,j);
                if (t >= 0) transponder[t].service[s].lcn = lcn;
                j+=2;
            }
            break;

        default:
            break;
    }
    return 2+descriptor_length;
}

int get_pat_info(unsigned char *buf)
{
    int transport_stream_id, service_id;
    int section_length, num_services;
    int i,j,t,s;

    j = 0;
    j++;
    section_length = 3 + b110(buf,j); j+=2;
    transport_stream_id = b150(buf,j); j+=2;
    t = add_transport_stream_id(transport_stream_id);
    j += 3;

    num_services = (section_length-8-4)/4;
    for (i=0; i<num_services; i++) {
        service_id = b150(buf,j); j+=2;
        if (service_id != 0) {
            s = add_service_id(t, service_id);
            transponder[t].service[s].pmt_pid = b120(buf,j);
        } else {
        }
        j+=2;
    }

    return num_services;
}


int get_nit_info(unsigned char *buf)
{
    int network_id, transport_stream_id, original_network_id;
    int section_length, descriptor_length;
    int j,k,t;

    j = 0;
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    network_id = b150(buf, j); j+=2;
    j += 3;
    descriptor_length = b110(buf, j); j+=2;

    // skip the descriptors   - do later ...
    k = j;
    while(j < (k+descriptor_length)) {
//        j += do_desc(&buf[j],-1,-1,-1);
        j += b70(buf, j+1);
        j += 2;
    }
    j += 2;
    while (j < (section_length-4)) {
        transport_stream_id = b150(buf, j); j+= 2;
        t = add_transport_stream_id(transport_stream_id);
        original_network_id = b150(buf, j); j+= 2;
        transponder[t].original_network_id = original_network_id;

        descriptor_length = b110(buf, j); j+= 2;
        k = j;
        while (j < (k+descriptor_length)) {
            j += do_desc(&buf[j],t,-1,-1);
        }
        transponder[t].network_id = network_id;
    }

    // do these now, so that we know the sat
    j = 8;
    descriptor_length = b110(buf, j); j+=2;
    k = j;
    while(j < (k+descriptor_length)) {
        j += do_desc(&buf[j],t,-1,-1);
    }

    return num_transponders;
}


int get_sdt_info(unsigned char *buf)
{
    int transport_stream_id, original_network_id, service_id;
    int section_length, descriptor_length;
    int j,k,t,s;
    int eit_schedule_flag,eit_present_following_flag,running_status,free_ca_mode;

    j = 0;
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    transport_stream_id = b150(buf, j); j+=2;
    t = add_transport_stream_id(transport_stream_id);
    j += 3;
    original_network_id = b150(buf, j); j+=2;
    j ++;
    while (j<(section_length-4)) {
        service_id                  = b150(buf, j); j+=2;
        s = add_service_id(t, service_id);
        eit_schedule_flag           = b1(buf, j);
        eit_present_following_flag  = b0(buf, j); j++;
        running_status              = b75(buf, j);
        free_ca_mode                = b4(buf, j);
        descriptor_length           = b110(buf, j); j+=2;

        k = j;
        while (j < (k+descriptor_length)) {
            j += do_desc(&buf[j],t,s,-1);
        }
    }

    return 0;
}


int get_pmt_info(unsigned char *buf, int t)
{
    int service_id, elementary_pid, stream_type, pcr_pid;
    int section_length, descriptor_length;
    int j,k,s;

    j = 0;
    j++;
    section_length = 3 + b110(buf, j); j+=2;
    service_id = b150(buf, j); j+=2;
    s = add_service_id(t, service_id);
    j += 3;
    pcr_pid = b120(buf, j); j+=2;
    transponder[t].service[s].pcr_pid = pcr_pid;
    descriptor_length = b110(buf, j); j+=2;
    k = j;
    while (j < (k+descriptor_length)) {
        j += do_desc(&buf[j],t,s,-1);
    }

    while (j<(section_length-4)) {
        stream_type           = b70(buf, j++);
        elementary_pid        = b120(buf, j); j+=2;
        switch(stream_type) {
        case 0x01:
        case 0x02:
            //video pid
            transponder[t].service[s].video_pid = elementary_pid;
            break;
        case 0x03:
        case 0x04:
            //audio pid
            if (transponder[t].service[s].audio_pid == -1)
            transponder[t].service[s].audio_pid = elementary_pid;
            break;
        case 0x05:
            break;
        case 0x06:
            //pes pid
            break;
        case 0x0a:
        case 0x0b:
        case 0x0c:
        case 0x0d:
        case 0x0e:
            //dsmcc pid
            break;
        case 0xc0: // patch
        case 0xc1: // patch
        case 0xc6: // program_map
        case 0xc8: // download s/w
            //s/w downlod pid
            break;
        default:
            //other data pid
            break;
        }
        descriptor_length     = b110(buf, j); j+=2;
        k = j;
        while (j < (k+descriptor_length)) {
            j += do_desc(&buf[j],t,s,stream_type);
        }
    }

    return 0;
}
