#ifndef _PARSE_DVB_H_
#define _PARSE_DVB_H_


extern int debug;

//----------------------------------------------------------------------------
typedef struct {
    int transponder_id;
    int lcn;
    int service_id;
    int service_type;
    char provider_name[0x40];
    char service_name[0x40];
    int pmt_pid;
    int video_pid;
    int audio_pid;
    int audio2_pid;
    int pcr_pid;
    int subtitles_pid;
    int teletext_pid;
    int dsmcc_pid;
    char logo[0x40];
} service_type;

typedef struct {
    int network_id;
    char network_name[0x40];
    int frequency;
    int bandwidth;
    int fec_high;
    int fec_low;
    int modulation;
    int transmission_mode;
    int guard_interval;
    int hierarchy;

    int original_network_id;
    int transport_stream_id;
    int num_services;
    service_type service[0x20];

} transponder_type;

extern int num_transponders;
extern transponder_type transponder[10];


//----------------------------------------------------------------------------

typedef struct {
    unsigned char  table_id                         : 8;
    unsigned char  section_syntax_indicator         : 1;
    unsigned char  zero                             : 1;
    unsigned char  reserved1                        : 2;
    unsigned short section_length                   :12;
    unsigned short transport_stream_id              :16;
    unsigned char  reserved2                        : 2;
    unsigned char  version_number                   : 5;
    unsigned char  current_next_indicator           : 1;
    unsigned char  section_number                   : 8;
    unsigned char  last_section_number              : 8;
} section_header_type;

#define b310(buf,j)  (((buf[j] << 24) & 0xff000000) + ((buf[j+1] << 16) & 0x00ff0000) + ((buf[j+2] << 8) & 0x0000ff00) + (buf[j+3] & 0x000000ff))
#define b150(buf,j)  (((buf[j] << 8) & 0xff00) + (buf[j+1] & 0x00ff))
#define b140(buf,j)  (((buf[j] << 8) & 0x7f00) + (buf[j+1] & 0x00ff))
#define b130(buf,j)  (((buf[j] << 8) & 0x3f00) + (buf[j+1] & 0x00ff))
#define b120(buf,j)  (((buf[j] << 8) & 0x1f00) + (buf[j+1] & 0x00ff))
#define b110(buf,j)  (((buf[j] << 8) & 0x0f00) + (buf[j+1] & 0x00ff))
#define b100(buf,j)  (((buf[j] << 8) & 0x0700) + (buf[j+1] & 0x00ff))
#define b90(buf,j)  (((buf[j] << 8) & 0x0300) + (buf[j+1] & 0x00ff))
#define b80(buf,j)  (((buf[j] << 8) & 0x0100) + (buf[j+1] & 0x00ff))

#define b210(buf,j)  (((buf[j] << 16) & 0x3f0000) + ((buf[j+1] << 8) & 0x00ff00) + ((buf[j+2] >> 0) & 0x0000ff))
#define b190(buf,j)  (((buf[j] << 16) & 0x0f0000) + ((buf[j+1] << 8) & 0x00ff00) + ((buf[j+2] >> 0) & 0x0000ff))
#define b174(buf,j)  (((buf[j] << 12) & 0x003000) + ((buf[j+1] << 4) & 0x000ff0) + ((buf[j+2] >> 4) & 0x00000f))
#define b152(buf,j)  (((buf[j] << 6) & 0x3fc0) + ((buf[j+1] >> 2) & 0x003f))

#define b70(buf,j)   ((buf[j] >> 0) & 0xff)
#define b60(buf,j)   ((buf[j] >> 0) & 0x7f)
#define b50(buf,j)   ((buf[j] >> 0) & 0x3f)
#define b40(buf,j)   ((buf[j] >> 0) & 0x1f)
#define b30(buf,j)   ((buf[j] >> 0) & 0x0f)
#define b20(buf,j)   ((buf[j] >> 0) & 0x07)
#define b10(buf,j)   ((buf[j] >> 0) & 0x03)

#define b51(buf,j)   ((buf[j] >> 1) & 0x01)
#define b76(buf,j)   ((buf[j] >> 6) & 0x03)
#define b75(buf,j)   ((buf[j] >> 5) & 0x07)
#define b74(buf,j)   ((buf[j] >> 4) & 0x0f)
#define b72(buf,j)   ((buf[j] >> 2) & 0x3f)
#define b63(buf,j)   ((buf[j] >> 3) & 0x0f)
#define b54(buf,j)   ((buf[j] >> 4) & 0x03)
#define b53(buf,j)   ((buf[j] >> 3) & 0x07)
#define b43(buf,j)   ((buf[j] >> 3) & 0x03)
#define b71(buf,j)   ((buf[j] >> 1) & 0x7f)
#define b21(buf,j)   ((buf[j] >> 1) & 0x03)

#define b7(buf,j)    ((buf[j] >> 7) & 0x01)
#define b6(buf,j)    ((buf[j] >> 6) & 0x01)
#define b5(buf,j)    ((buf[j] >> 5) & 0x01)
#define b4(buf,j)    ((buf[j] >> 4) & 0x01)
#define b3(buf,j)    ((buf[j] >> 3) & 0x01)
#define b2(buf,j)    ((buf[j] >> 2) & 0x01)
#define b1(buf,j)    ((buf[j] >> 1) & 0x01)
#define b0(buf,j)    ((buf[j] >> 0) & 0x01)


// descriptor tag's
#define VIDEO_STREAM_DESCRIPTOR                 0x0002
#define AUDIO_STREAM_DESCRIPTOR                 0x0003
#define HIERARCHY_DESCRIPTOR                    0x0004
#define REGISTRATION_DESCRIPTOR                 0x0005
#define DATA_STREAM_ALIGNMENT_DESCRIPTOR        0x0006
#define TARGET_BACKGROUND_GRID_DESCRIPTOR       0x0007
#define VIDEO_WINDOW_DESCRIPTOR                 0x0008
#define CA_DESCRIPTOR                           0x0009
#define ISO_639_LANGUAGE_DESCRIPTOR             0x000a
#define SYSTEM_CLOCK_DESCRIPTOR                 0x000b
#define MULTIPLEX_BUFFER_UTILISATION_DESCRIPTOR 0x000c
#define COPYRIGHT_DESCRIPTOR                    0x000d
#define MAXIMUM_BITRATE_DESCRIPTOR              0x000e
#define PRIVATE_DATA_INDICATOR_DESCRIPTOR       0x000f
#define SMOOTHING_BUFFER_DESCRIPTOR             0x0010
#define STD_DESCRIPTOR                          0x0011
#define IBP_DESCRIPTOR                          0x0012
#define CAROUSEL_IDENTIFIER_DESCRIPTOR          0x0013
#define ASSOCIATION_TAG_DESCRIPTOR              0x0014
#define DEFERRED_ASSOCIATION_TAGS_DESCRIPTOR    0x0015

#define NETWORK_NAME_DESCRIPTOR                 0x0040
#define SERVICE_LIST_DESCRIPTOR                 0x0041
#define STUFFING_DESCRIPTOR                     0x0042
#define SATELLITE_DELIVERY_SYSTEM_DESCRIPTOR    0x0043
#define CABLE_DELIVERY_SYSTEM_DESCRIPTOR        0x0044
#define BOUQUET_NAME_DESCRIPTOR                 0x0047
#define SERVICE_DESCRIPTOR                      0x0048
#define COUNTRY_AVAILABILITY_DESCRIPTOR         0x0049
#define LINKAGE_DESCRIPTOR                      0x004a
#define NVOD_REFERENCE_DESCRIPTOR               0x004b
#define TIME_SHIFTED_SERVICE_DESCRIPTOR         0x004c
#define SHORT_EVENT_DESCRIPTOR                  0x004d
#define EXTENDED_EVENT_DESCRIPTOR               0x004e
#define TIME_SHIFTED_EVENT_DESCRIPTOR           0x004f
#define COMPONENT_DESCRIPTOR                    0x0050
#define MOSAIC_DESCRIPTOR                       0x0051
#define STREAM_IDENTIFIER_DESCRIPTOR            0x0052
#define CA_IDENTIFIER_DESCRIPTOR                0x0053
#define CONTENT_DESCRIPTOR                      0x0054
#define PARENTAL_RATING_DESCRIPTOR              0x0055
#define TELETEXT_DESCRIPTOR                     0x0056
#define TELEPHONE_DESCRIPTOR                    0x0057
#define LOCAL_TIME_OFFSET_DESCRIPTOR            0x0058
#define SUBTITLING_DESCRIPTOR                   0x0059
#define TERRESTRIAL_DELIVERY_SYSTEM_DESCRIPTOR  0x005a
#define MULTILINGUAL_NETWORK_NAME_DESCRIPTOR    0x005b
#define MULTILINGUAL_BOUQUET_NAME_DESCRIPTOR    0x005c
#define MULTILINGUAL_SERVICE_NAME_DESCRIPTOR    0x005d
#define MULTILINGUAL_COMPONENT_DESCRIPTOR       0x005e
#define PRIVATE_DATA_SPECIFIER_DESCRIPTOR       0x005f
#define SERVICE_MOVE_DESCRIPTOR                 0x0060
#define SHORT_SMOOTHING_BUFFER_DESCRIPTOR       0x0061
#define FREQUENCY_LIST_DESCRIPTOR               0x0062
#define PARTIAL_TRANSPORT_STREAM_DESCRIPTOR     0x0063
#define DATA_BROADCAST_DESCRIPTOR               0x0064
#define CA_SYSTEM_DESCRIPTOR                    0x0065
#define DATA_BROADCAST_ID_DESCRIPTOR            0x0066
#define TRANSPORT_STREAM_DESCRIPTOR             0x0067
#define DSNG_DESCRIPTOR                         0x0068
#define PDC_DESCRIPTOR                          0x0069
#define AC3_DESCRIPTOR                          0x006a
#define ANCILLARY_DESCRIPTOR                    0x006b
#define CELL_LIST_DESCRIPTOR                    0x006c
#define CELL_FREQUENCY_LINK_DESCRIPTOR          0x006d
#define ANNOUNCEMENT_SUPPORT_DESCRIPTOR         0x006e
//
#define USER_DESCRIPTOR                         0x0080
#define CHANNEL_MAP_DESCRIPTOR                  0x0083 // OD, in NIT

//----------------------------------------------------------------------------
#define to_printable(c) (((c >= 0x20) && (c<0x80)) ? c : ' ')

//----------------------------------------------------------------------------
extern EnumType stream_type_str[];
extern EnumType service_type_str[];
extern EnumType bandwidth_str[];
extern EnumType constellation_str[];
extern EnumType hierarchy_str[];
extern EnumType code_rate_str[];
extern EnumType guard_interval_str[];
extern EnumType transmission_mode_str[];


extern void dump(void *buf, int len);
extern unsigned char *dvb_enumtype(EnumType *qenum, int id) ;
extern int to_name(char *dest_str, char *source_str, int len);
extern int add_transport_stream_id(int transport_stream_id);
extern int add_service_id(int t, int service_id);
extern int do_desc(unsigned char *pkt, int t, int s, int stream_type);
extern int get_pat_info(unsigned char *buf);
extern int get_nit_info(unsigned char *buf);
extern int get_sdt_info(unsigned char *buf);
extern int get_pmt_info(unsigned char *buf, int t);

#endif
