//
//TRANSFORM AN XML/XSL PAIR TO HTML
//
#include <windows.h>
#include <stdio.h>
#include <string.h>
#import <msxml4.dll> raw_interfaces_only
using namespace MSXML2;

// Macro that calls a COM method returning HRESULT value:
#define HRCALL(a, errmsg) \
do { \
    hr = (a); \
    if (FAILED(hr)) { \
        dprintf( "%s:%d  HRCALL Failed: %s\n  0x%.8x = %s\n", \
            __FILE__, __LINE__, errmsg, hr, #a ); \
        goto clean; \
    } \
} while (0)

// Helper function that puts output in stdout and debug window in Visual Studio:
void dprintf(char * format, ...)
{
    static char buf[1024];
    va_list args;
    va_start(args, format);
    vsprintf(buf, format, args);
    va_end(args);
    OutputDebugStringA(buf);
    printf("%s", buf);
}

// Helper function to create a DOM instance:
MSXML2::IXMLDOMDocument * DomFromCOM()
{
    HRESULT hr;
    MSXML2::IXMLDOMDocument *pxmldoc = NULL;

    HRCALL(CoCreateInstance(__uuidof(DOMDocument40),
        NULL,
        CLSCTX_INPROC_SERVER,
        __uuidof(MSXML2::IXMLDOMDocument),
        (void**)&pxmldoc),
        "Create a new DOMDocument");

    HRCALL(pxmldoc->put_async(VARIANT_FALSE), "should never fail");
    HRCALL(pxmldoc->put_validateOnParse(VARIANT_FALSE), "should never fail");
    HRCALL(pxmldoc->put_resolveExternals(VARIANT_FALSE), "should never fail");

    return pxmldoc;
clean:
    if (pxmldoc) {
        pxmldoc->Release();
    }
    return NULL;
}


void usage()
{
    printf("\nxml2html v0.01");
    printf("\n==============");
    printf("\n");
    printf("\nusage: xml2html <input file> <output file>");
    printf("\n         -i    input filename");
    printf("\n         -o    output filename");

}

int main(int argc, char* argv[])
{
    int i,j;
    int numin;
    char ifilename[0x100] = "channels";
    char ofilename[0x100] = "channels";
    char cstr[0x100];
    FILE *Op;
    BSTR ustr = SysAllocStringLen(NULL, 0x200);
    MSXML2::IXMLDOMDocument *pXMLDom=NULL;
    MSXML2::IXMLDOMDocument *pXSLDoc=NULL;
    MSXML2::IXMLDOMParseError *pXMLErr=NULL;
    IDispatch *pDisp=NULL;
    BSTR bstr = NULL;
    long ecode;
    VARIANT_BOOL status;
    VARIANT var;
    HRESULT hr;

    j = 0;
    for(i=1; i<argc; i++) {
        if (argv[i][0] == '-') {
            switch (argv[i][1]) {
                case 'i' :
                    sscanf(argv[++i],"%[^]]", ifilename);
                    j = 1;
                    break;
                case 'o' :
                    sscanf(argv[++i],"%[^]]", ofilename);
                    j = 2;
                    break;
                case '?' :
                default  :
                    usage();
                    return 1;
            }
        }
        else {
            if (j == 0) {
                numin = sscanf(argv[i],"%[^]]", ifilename);
                j++;
            }
            else
            if (j == 1) {
                numin = sscanf(argv[i],"%[^]]", ofilename);
                j++;
            }
        }
    }
    printf("\nxml2html v0.01");
    printf("\n==============");
    printf("\n");
    printf("\nifilename  = %s.xml", ifilename);
    printf("\nofilename  = %s.htm", ofilename);
    printf("\n");


    CoInitialize(NULL);

    //load the .xml into a dom
    pXMLDom = DomFromCOM();
    if (!pXMLDom) goto clean;
    VariantInit(&var);
    strcpy(cstr, ifilename);
    strcat(cstr, ".xml");
    MultiByteToWideChar(CP_UTF8, 0, cstr, 0x100, ustr, 0x200);
    V_BSTR(&var) = SysAllocString(ustr); //L"mp3.xml");
    V_VT(&var) = VT_BSTR;
    HRCALL(pXMLDom->load(var,&status),"xml->load(): ");
    if (status!=VARIANT_TRUE) {
        HRCALL(pXMLDom->get_parseError(&pXMLErr), "xml->get_parseError: ");
        HRCALL(pXMLErr->get_reason(&bstr), "parseError->get_reason: ");
        dprintf("Failed to load DOM from %s. %S\n", cstr, bstr);
        goto clean;
    }

    //load the .xsl into a dom
    pXSLDoc = DomFromCOM();
    if (!pXSLDoc) goto clean;
    VariantClear(&var);
    strcpy(cstr, ifilename);
    strcat(cstr, ".xsl");
    MultiByteToWideChar(CP_UTF8, 0, cstr, 0x100, ustr, 0x200);
    V_BSTR(&var) = SysAllocString(ustr); //L"mp3.xsl");
    V_VT(&var) = VT_BSTR;
    HRCALL(pXSLDoc->load(var,&status), "xsl->load(): ");
    if (status != VARIANT_TRUE) {
        HRCALL(pXMLDom->get_parseError(&pXMLErr), "xsl->get_parseError: ");
        HRCALL(pXMLErr->get_reason(&bstr), "parseError->get_reason: ");
        dprintf("Failed to load DOM from %s. %S\n", cstr, bstr);
        goto clean;
    }

    // Transform dom to a string:
    HRCALL(pXMLDom->transformNode(pXSLDoc, &bstr), "");
    HRCALL(pXMLDom->get_parseError(&pXMLErr), "");
    HRCALL(pXMLErr->get_errorCode(&ecode), "");
    if (ecode != 0) {
        if (bstr) SysFreeString(bstr);
        HRCALL(pXMLErr->get_reason(&bstr),"");
        dprintf("Failed to transformNode:\n%S\n", bstr);
        goto clean;
    }

    //output the html
    strcpy(cstr, ofilename);
    strcat(cstr, ".htm");
    Op = fopen(cstr, "wb");
    if (Op) {
        for (i=0; i<SysStringLen(bstr); i++) {
            fprintf(Op, "%c", bstr[i]);
        }
        fclose(Op);
    }

clean:
   if (ustr) SysFreeString(ustr);
   if (bstr) SysFreeString(bstr);
   if (&var) VariantClear(&var);
   if (pXMLErr) {pXMLErr->Release();}
   if (pXMLDom) {pXMLDom->Release();}
   if (pXSLDoc) {pXSLDoc->Release();}

   CoUninitialize();
   return 0;
}
